﻿CompilerIf #PB_Compiler_Version < 530
  IncludePath "binaries"
CompilerElse
  IncludePath "../binaries"
CompilerEndIf

Enumeration OEM
  #OEM_TESSERACT_ONLY           ; Run Tesseract only - fastest
  #OEM_CUBE_ONLY				        ; Run Cube only - better accuracy, but slower
  #OEM_TESSERACT_CUBE_COMBINED  ; Run both and combine results - best accuracy
  #OEM_DEFAULT				          ; 
EndEnumeration
	
Enumeration PageSegMode
  #PSM_OSD_ONLY                 ; Orientation and script detection only.
  #PSM_AUTO_OSD		              ; Automatic page segmentation with orientation and script detection. (OSD)
  #PSM_AUTO_ONLY		            ; Automatic page segmentation, but no OSD, or OCR.
  #PSM_AUTO			                ; Fully automatic page segmentation, but no OSD.
  #PSM_SINGLE_COLUMN	          ; Assume a single column of text of variable sizes.
  #PSM_SINGLE_BLOCK_VERT_TEXT   ; Assume a single uniform block of vertically aligned text.
  #PSM_SINGLE_BLOCK			        ; Assume a single uniform block of text. (Default.)
  #PSM_SINGLE_LINE			        ; Treat the image as a single text line.
  #PSM_SINGLE_WORD			        ; Treat the image as a single word.
  #PSM_CIRCLE_WORD			        ; Treat the image as a single word in a circle.
  #PSM_SINGLE_CHAR			        ; Treat the image as a single character.
  #PSM_SPARSE_TEXT			        ; Find as much text as possible in no particular order.
  #PSM_SPARSE_TEXT_OSD		      ; Sparse text with orientation and script det.
  #PSM_RAW_LINE				          ; Treat the image as a single text line, bypassing hacks that are Tesseract-specific.		
  #PSM_COUNT                    ; Number of enum entries.
EndEnumeration

Enumeration PageIteratorLevel 
  #RIL_BLOCK                    ; Block of text/image/separator line.
  #RIL_PARA                     ; Paragraph within a block.
  #RIL_TEXTLINE                 ; Line within a paragraph.
  #RIL_WORD                     ; Word within a textline.
  #RIL_SYMBOL                   ; Symbol/character within a word.
EndEnumeration

#L_INSERT     = 0               ; stuff it in; no copy, clone or copy-clone
#L_COPY       = 1               ; make/use a copy of the object
#L_CLONE      = 2               ; make/use clone (ref count) of the object
#L_COPY_CLONE = 3               ; make a new object And fill With With clones of each object in the Array(s)

; From Leptonica 
; pix.h
#PIXAA_VERSION_NUMBER = 2
#PIXA_VERSION_NUMBER  = 2
#BOXA_VERSION_NUMBER  = 2
#BOXAA_VERSION_NUMBER = 3

Structure PIX
	w.l                           ; width in pixels                   
	h.l			                      ; height in pixels                  
	d.l			                      ; depth in bits (bpp)               
	spp.l		                      ; number of samples per pixel       
	wpl.l		                      ; 32-bit words/line                 
				                        ;     refcount.l  ;< reference count (1 if no clones)  
	xres.l		                    ; image res (ppi) in x direction    < (use 0 if unknown)  
	yres.l		                    ; image res (ppi) in y direction    < (use 0 if unknown)    	
	informat.l                    ; input file format, IFF_*
	special.l	                    ; special instructions for I/O, etc 
	*text.Ascii	                  ; text string associated with pix   
	*colormap	                    ; PixColormap  ;< colormap (may be null)            
	*Data.Long	                  ; the image data                    
EndStructure

Structure PIXA	
  n.l  
  nalloc.l
  refcount.l
  *pixPtrArray
  *boxaPtrArray
EndStructure	
 
Structure BOX
	x.l                   
	y.l                  
	w.l               
	h.l       
	refcount.l		; reference count (1 if no clones)                                     
EndStructure

Structure BOXA
	n.l           ; number of box in ptr array                   
	nalloc.l			; number of box ptrs allocated                                    
	refcount.l		; box ptr array           
	*boxPtrArray 
EndStructure

ImportC "libtesseract304.lib"
  TessBaseAPIClear(*TessBaseAPI)
  TessBaseAPICreate()
  TessBaseAPIDelete(*TessBaseAPI)
  TessBaseAPIEnd(*TessBaseAPI)
  TessBaseAPIGetUTF8Text(*TessBaseAPI)
  TessBaseAPIInit1(*TessBaseAPI, datapath.p-AscII, language.p-AscII, OEM, *array_configs, configs_size)
  TessBaseAPIInit2(*TessBaseAPI, datapath.p-AscII, language.p-AscII, OEM)
  TessBaseAPIInit3(*TessBaseAPI, datapath.p-AscII, language.p-AscII)
  TessBaseAPIRecognize(*TessBaseAPI, monitor_class)
  TessBaseAPISetPageSegMode(*TessBaseAPI, TessPageSegMode)
  TessBaseAPISetImage.l(*TessBaseAPI, *imagedata, width.l, height.l, bytes_per_pixel.l, bytes_per_line.l)      
  TessBaseAPISetImage2(*TessBaseAPI, *PIX)  
  TessBaseAPIRect.l(*TessBaseAPI, *imagedata, bytes_per_pixel.l, bytes_per_line.l, left.l, top.l, width.l, height.l)
  TessBaseAPIGetComponentImages(*TessBaseAPI, TessPageIteratorLevel, text_only, *pixaPtrArray, *blockids)   
  TessBaseAPISetRectangle(*TessBaseAPI, left.l, top.l, width.l, height.l)
  TessBaseAPIMeanTextConf.l(*TessBaseAPI)
	LeptPixRead(filename.p-Ascii)
	LeptPixDestroy(hPix)
	LeptPixWritePNG(filename.p-Ascii, hPix, gamma.f)
	LeptPixReadMemPNG(*hMemory, size)  		
	EndImport		
	
ImportC "liblept168.lib"		
    boxaGetBox(*boxa, index.l, accessflag.l);    
EndImport
	
Procedure.s ToAscii(*str)
	If *str 
		Protected str$ = PeekS(*str, #PB_Default, #PB_Unicode)
		Protected out$ = Space(1 + Len(str$) / SizeOf(Character))					
		PokeS(@out$, str$, #PB_Default, #PB_Ascii)
		ProcedureReturn out$
	EndIf
EndProcedure

Procedure TesseractInit(PageSegMode, OEM, dictionarypath.s, language.s, tesscfg.s)
  Protected *hApi = TessBaseAPICreate()
  Protected CFG$ = ToAscii(@tesscfg)
  Protected CFG = @CFG$				
	If Not *hApi
		Debug "Failed to create Tesseract "
		ProcedureReturn 0
	EndIf		
	TessBaseAPISetPageSegMode(*hApi, PageSegMode) 
	If TessBaseAPIInit1(*hApi, dictionarypath, language, OEM, @CFG, 1)	
		Debug Str(TessBaseAPIInit1) + "Failed to init Tesseract!"
		TessBaseAPIEnd(*hApi)
		ProcedureReturn 0
	EndIf
	
	ProcedureReturn *hApi
EndProcedure

Procedure$ cvIplImage2TextOCR(*hApi, *image0)
  Protected *imgcopy.IplImage, *gray.IplImage, *bin.IplImage, Result$
  
  *imgcopy = cvCloneImage(*image0)
  *gray = cvCreateImage(*imgcopy\width, *imgcopy\height, #IPL_DEPTH_8U, 1)
  *bin = cvCreateImage(*imgcopy\width, *imgcopy\height, #IPL_DEPTH_8U, 1)
  If *imgcopy\nChannels = 3
    cvCvtColor(*imgcopy, *gray, #CV_BGR2GRAY, 1)
  Else
    *gray = cvCloneImage(*imgcopy)
  EndIf  
  threshold.d = cvThreshold(*gray, *bin, 10, 255, #CV_THRESH_OTSU)  
  
  TessBaseAPISetImage(*hApi, *bin\imageData, *bin\width, *bin\height, 1, *bin\widthStep)
  TessBaseAPIRect(*hApi, *bin\imageData, 1, *bin\widthStep, 120, 0, 320, 150);*bin\width, *bin\height)	
  If TessBaseAPIRecognize(*hApi, 0)
    Result$ = "Recognition error"
  Else
    Result$ = RTrim(PeekS(TessBaseAPIGetUTF8Text(*hApi), -1, #PB_UTF8),Chr(10))
  EndIf
  TessBaseAPIClear(*hApi)
  
  ProcedureReturn Result$
EndProcedure

Procedure$ cvIplImageRect2TextOCR(*hApi, *image0, rectX, rectY, rectWidth, rectHeight)
  Protected *imgcopy.IplImage, *gray.IplImage, *bin.IplImage, Result$
  
  *imgcopy = cvCloneImage(*image0)
  *gray = cvCreateImage(*imgcopy\width, *imgcopy\height, #IPL_DEPTH_8U, 1)
  *bin = cvCreateImage(*imgcopy\width, *imgcopy\height, #IPL_DEPTH_8U, 1)
  If *imgcopy\nChannels = 3
    cvCvtColor(*imgcopy, *gray, #CV_BGR2GRAY, 1)
  Else
    *gray = cvCloneImage(*imgcopy)
  EndIf  
  threshold.d = cvThreshold(*gray, *bin, 10, 255, #CV_THRESH_OTSU)  
  
  TessBaseAPISetImage(*hApi, *bin\imageData, *bin\width, *bin\height, 1, *bin\widthStep)
  TessBaseAPIRect(*hApi, *bin\imageData, 1, *bin\widthStep, rectX, rectY, rectWidth, rectHeight);*bin\width, *bin\height)	
  If TessBaseAPIRecognize(*hApi, 0)
    Result$ = "Recognition error"
  Else
    Result$ = RTrim(PeekS(TessBaseAPIGetUTF8Text(*hApi), -1, #PB_UTF8),Chr(10))
  EndIf
  TessBaseAPIClear(*hApi)
  
  ProcedureReturn Result$
EndProcedure
; IDE Options = PureBasic 5.43 LTS (Windows - x86)
; CursorPosition = 52
; FirstLine = 23
; Folding = w
; EnableUnicode
; EnableXP